//*****************************************************************************
// File Name            : camera_mux.v
//-----------------------------------------------------------------------------
// Function             : camera_mux 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo@himwari_co 
//-----------------------------------------------------------------------------
// History
// -.-- 2017/7/25
//*****************************************************************************
module camera_mux (
	CLK,
	RSTB,
	SIO_D,
	SIO_C,
    XCLK,PCLK,VSYNC,HREF,D,
    DCLK_M,
	//da_r,da_g,da_b,
	DATA_M,
	VSD_M,HSD_M,DEN_M,
	LED
	);	

  input		CLK;
  input		RSTB;
  inout		SIO_C;
  inout		SIO_D;
  output XCLK;  //camera system clock
  input PCLK;
  input VSYNC;
  input HREF;
  input [9:2]D;
  output    DCLK_M;
  /*
  output [1:0]da_r;
  output [1:0]da_g;
  output [1:0]da_b;
  */
  output [3:0]DATA_M;
  output VSD_M;
  output HSD_M;  
  output DEN_M;
	output LED;
		
	wire		sccb_sccb_sda_oe;
	wire		sccb_sccb_scl_oe;
	reg clk_reg;
	reg [3:0]cnt;
	wire dclk;
	
  wire h_c_en;
  wire [9:0] v_c;
  wire [9:0] h_c;
  wire [7:0]da_r_tmp;
  wire [7:0]da_g_tmp;
  wire [7:0]da_b_tmp;
  wire da_en_tmp;
  wire hsync_tmp;
  wire vsync_tmp;
  //varm
  wire  wea;
  wire [13 : 0] addra;
  wire [15 : 0] dina;
  wire [13 : 0] addrb;
  wire [15 : 0] doutb;
  //vout
  wire in_a_da_en;
  wire [7:0]in_a_da_r;
  wire [7:0]in_a_da_g;
  wire [7:0]in_a_da_b;
	
   always@(posedge CLK)
	   clk_reg = ~clk_reg;
	
	assign XCLK = clk_reg;

   always@(posedge CLK)
	   cnt =cnt + 4'd1;

	assign led = cnt[3];	
		
	assign dclk = cnt[1];
	
	nios2e_sys nios2e_sys(
		.clk_clk      (CLK),       //   clk.clk
		.reset_reset_n (RSTB), // reset.reset_n
		.sccb_sda_in   (SIO_D),   //  sccb.sda_in
		.sccb_scl_in   (SIO_C),   //      .scl_in
		.sccb_sda_oe   (sccb_sda_oe),   //      .sda_oe
		.sccb_scl_oe   (sccb_scl_oe)    //      .scl_oe
	);
	
   assign SIO_D = (sccb_sda_oe==1'b1)?1'b0:1'bz;
   assign SIO_C = (sccb_scl_oe==1'b1)?1'b0:1'bz;

camera_if camera_if(
  .pclk(     PCLK),           
  .c_vsync(  VSYNC), 
  .href(     HREF),           
  .data(     D),    
  .dclk(     dclk),        
  .rstb(     RSTB), 
  .h_c_en(h_c_en),  
  .v_c(v_c),
  .h_c(h_c),             
  .gen_da_en(in_a_da_en),
  .gen_da_r( in_a_da_r), 
  .gen_da_g( in_a_da_g), 
  .gen_da_b( in_a_da_b), 
  .wea(wea),     
  .addra(addra), 
  .dina( dina),  
  .addrb(addrb), 
  .doutb(doutb) 
);

//vram_128_128 vram (
vram_128_96 vram (
        .wrclock(PCLK),
        .wren(wea), // Bus [0 : 0] 
        .wraddress(addra), // Bus [13 : 0] 
        .data(dina), // Bus [15 : 0] 
        .rdclock(dclk),
        .rdaddress(addrb), // Bus [13 : 0] 
        .q( doutb));// Bus [15 : 0] 
		
lcd_ctrl lcd_ctrl (
  .clk(         dclk),           
  .rstb(        RSTB),     
  .h_c_en(   h_c_en),     
  .out_hsync( hsync_tmp),   
  .out_vsync( vsync_tmp),   
  .v_c(         v_c),           
  .h_c(         h_c),  
  .in_a_da_en(  in_a_da_en),      
  .in_a_da_r(   in_a_da_r),       
  .in_a_da_g(   in_a_da_g),       
  .in_a_da_b(   in_a_da_b),  
/*  
  .in_b_da_en(  in_b_da_en),      
  .in_b_da_r(   in_b_da_r),       
  .in_b_da_g(   in_b_da_g),       
  .in_b_da_b(   in_b_da_b),      
  .in_c_da_en(  in_c_da_en),      
  .in_c_da_r(   in_c_da_r),       
  .in_c_da_g(   in_c_da_g),       
  .in_c_da_b(   in_c_da_b),      
  .in_d_da_en(  in_d_da_en),      
  .in_d_da_r(   in_d_da_r),       
  .in_d_da_g(   in_d_da_g),       
  .in_d_da_b(   in_d_da_b),    
*/  
  .out_da_r(    da_r_tmp),      
  .out_da_g(    da_g_tmp),      
  .out_da_b(    da_b_tmp),      
  .out_da_en(   da_en_tmp)        
);


  rgb_mux rgb_mux(
  .clk(    CLK),
  .rstb(   RSTB),
  .hsync(  hsync_tmp),
  .vsync(  vsync_tmp),
  .r_da(   da_r_tmp[7:3]),
  .g_da(   da_g_tmp[7:2]),
  .b_da(   da_b_tmp[7:3]),
  .den(    da_en_tmp),
  .dclk_m( DCLK_M),
  .hsync_m(HSD_M),
  .vsync_m(VSD_M),
  /*
  .r_da_m( da_r),
  .g_da_m( da_g),
  .b_da_m( da_b ),
  */
  .da_m( DATA_M),
  .den_m(  DEN_M)
  );
  
endmodule