//*****************************************************************************
// File Name            : max10_lcd_mux.v
//-----------------------------------------------------------------------------
// Function             : max10 lcd mux
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo@himwari_co 
//-----------------------------------------------------------------------------
// History
// -.-- 2017/7/25
//*****************************************************************************
module max10_lcd_mux(
    input CLK,
    input RSTB,
	output DCLK_M,
    output VSD_M,
    output HSD_M,
    output [3:0]DATA_M,
	output DEN_M,
	 //output LCD_LED,
	 input KEY1,
	 output LED_0
    );

  parameter hsync_cnt_max = 927;
  parameter hsync_fp = 39;
  parameter hsync_pw = 20;
  parameter hsync_bp = 88;
  parameter vsync_cnt_max = 524;
  parameter vsync_fp = 12;
  parameter vsync_pw = 3;
  parameter vsync_bp = 32;

  wire d_clk;
  wire RST;
  reg tmg_da_en;
  reg tmg_sync;
  reg [9:0] vsync_cnt;
  reg [10:0] hsync_cnt;
  reg [9:0]  v_p;
  reg [9:0]  h_p;
  reg [7:0] r_data;
  reg [7:0] g_data;
  reg [7:0] b_data;
  wire [7:0] r_data_e;
  wire [7:0] g_data_e;
  wire [7:0] b_data_e;
  
  reg [7:0]led_cnt ;
  reg vsync_ff;
  reg hsync_ff;
  wire clk_48m;
  wire locked_sig;
  reg [1:0] clk_cnt;
/*
clk_gen	clk_gen_inst (
	.areset ( RST ),
	.inclk0 ( CLK),
	.c0 ( clk_48m ),
	.c1 ( d_clk ),
	.locked ( locked_sig )
	);
*/


always @ (posedge CLK or negedge RSTB )
if (RSTB==1'b0) 
  clk_cnt <=2'b00;
else
  clk_cnt <= clk_cnt + 2'b01;
  
assign clk_48m = CLK;
assign d_clk = clk_cnt[1];  
assign RST = ~RSTB;
  
always @ (posedge d_clk or posedge RST )
if (RST==1'b1) 
  begin  
    vsync_cnt <= 10'd0;
    hsync_cnt <= 11'd0;
  end
else
    if (hsync_cnt == hsync_cnt_max)
	  begin
        hsync_cnt <= 11'd0;
	    if (vsync_cnt == vsync_cnt_max)
          vsync_cnt <= 10'd0;
        else
          vsync_cnt <= vsync_cnt + 10'd1;
	  end
    else
	  begin
        hsync_cnt <= hsync_cnt +11'd1;
        vsync_cnt <= vsync_cnt ;
	  end 

always @ (posedge d_clk or posedge RST )
if (RST==1'b1) 
  begin 
	tmg_sync  <= 1'b1; 
    vsync_ff<= 1'b1;
    hsync_ff <= 1'b1;
    tmg_da_en <= 1'b0;
  end
else
  begin
	if ((vsync_cnt==10'd0)&&(hsync_cnt==11'd0))
	  tmg_sync  <= 1'b1;
    else 
	  tmg_sync  <= 1'b0;
	//VSYNC
	if ((vsync_cnt==vsync_fp)&&(hsync_cnt==hsync_fp))
	  vsync_ff<= 1'b0;
	else
	  if ((vsync_cnt==(vsync_fp+vsync_pw))&&(hsync_cnt==hsync_fp))
	    vsync_ff<= 1'b1;
	  else
	    vsync_ff<= vsync_ff;
	//HSYNC
	if (hsync_cnt==hsync_fp)
	    hsync_ff <= 1'b0;
   else
	 if  (hsync_cnt==(hsync_fp+hsync_pw))
		 hsync_ff <= 1'b1;
	 else
		 hsync_ff <= hsync_ff;
	//
	if ((vsync_cnt>=(vsync_fp+vsync_bp+1))&&(vsync_cnt <=vsync_cnt_max))
     if (hsync_cnt==(hsync_fp+hsync_bp))
       tmg_da_en <= 1'b1;
	  else
		  if  (hsync_cnt==hsync_cnt_max)
		    tmg_da_en <= 1'b0;
	     else
		    tmg_da_en <= tmg_da_en;
	else
		tmg_da_en <= 1'b0;
  end


  
always @ (posedge d_clk or posedge RST )
if (RST==1'b1) 
  begin
    v_p <=10'd0;
    h_p <=10'd0;
  end
 else
   if(tmg_sync==1'b1)
      begin
        v_p <=10'd0;
        h_p <=10'd0;
      end
	else
	   if  ((vsync_cnt>=(vsync_fp+vsync_bp+1))&&(vsync_cnt <=  vsync_cnt_max))
         begin
            if (hsync_cnt== hsync_cnt_max)
				  v_p <= v_p + 10'd1;
				else
				  v_p <= v_p;
            if (hsync_cnt==hsync_cnt_max)
				  h_p <= 10'd0;
				else
				  if((hsync_cnt>(hsync_fp+hsync_bp))&&(hsync_cnt<=hsync_cnt_max))
				  h_p <= h_p + 10'd1;
			end
		else
        begin
          v_p <=10'd0;
          h_p <=10'd0;
        end
				  
always @ (posedge d_clk or posedge RST )
if (RST==1'b1) 
  begin
    r_data <=8'h00;  
    g_data <=8'h00;  
    b_data <=8'h00;  
  end
else
  if( tmg_da_en==1'b1)
    if( v_p <10'd240)
      if(h_p<10'h100)
		  begin
          r_data <=r_data + 8'h01;  
          g_data <=8'h00;  
          b_data <=8'h00;
       end			 
      else if(h_p<10'h200)
		  begin
          r_data <=8'h00;  
          g_data <=g_data + 8'h01;  
          b_data <=8'h00;
       end			  
      else if(h_p<10'h300)
		  begin
          r_data <=8'h00;  
          g_data <=8'h00;  
          b_data <=b_data + 8'h01;
       end			  
      else 
		  begin
          r_data <=r_data + 8'h01;  
          g_data <=g_data + 8'h01;
          b_data <=b_data + 8'h01;
       end	
    else	
      if(h_p<10'h100)
		  begin
          r_data <=r_data + 8'h01;  
          g_data <=g_data + 8'h01;
          b_data <=b_data + 8'h01;
       end			 
      else if(h_p<10'h200)
		  begin
          r_data <=r_data + 8'h01;  
          g_data <=g_data + 8'h01;  
          b_data <=8'h00;
       end			  
      else if(h_p<10'h300)
		  begin
          r_data <=8'h00;  
          g_data <=g_data + 8'h01;  
          b_data <=b_data + 8'h01;
       end			  
      else 
		  begin
          r_data <=r_data + 8'h01;  
          g_data <=8'h00;  
          b_data <=b_data + 8'h01;
       end		 
  else
    begin
      r_data <=8'h00;  
      g_data <=8'h00;  
      b_data <=8'h00;  
	 end
  
    

  //assign R_DA =(tmg_da_en==1'b1)?r_data[7:6]:2'b00;
  //assign G_DA =(tmg_da_en==1'b1)?g_data[7:6]:2'b00;
  //assign B_DA =(tmg_da_en==1'b1)?b_data[7:6]:2'b00;
  //assign HSYNC = hsync_ff;
  //assign VSYNC = vsync_ff;
  //assign DE = tmg_da_en;
 
  

  rgb_mux rgb_mux(
  .clk(    clk_48m),
  .rstb(   RSTB),
  .hsync(  hsync_ff),
  .vsync(  vsync_ff),

  .r_da(   r_data[7:3]), //fpga
  .g_da(   g_data[7:2]),
  .b_da(   b_data[7:3]),
  
  /*
  .r_da(   r_data[4:0]), //sim
  .g_da(   g_data[5:0]),
  .b_da(   b_data[4:0]),
  */
  .den(    tmg_da_en),
  .dclk_m( DCLK_M),
  .hsync_m(HSD_M),
  .vsync_m(VSD_M),
  .da_m(   DATA_M),
  .den_m(  DEN_M)
  );
  
always @ (posedge d_clk or posedge RST )
if (RST==1'b1) 
  begin  
    led_cnt <= 8'd0;
  end
else
    if ((hsync_cnt == hsync_cnt_max)&&(vsync_cnt == vsync_cnt_max))
	  begin
          led_cnt <= led_cnt + 8'd1;
	  end
    else
	  begin
        led_cnt <= led_cnt ;
	  end
	  
 assign LED_0 = (KEY1==1'b1)?led_cnt[7]:1'b1;
 
endmodule