//*****************************************************************************
// File Name            : rgb_dmux.v
//-----------------------------------------------------------------------------
// Function             : rgb_dmux
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2017/7/25
// -.-- 2017/9/01
//*****************************************************************************
module rgb_mux(
    input clk,
    input rstb,
    input vsync,
    input hsync,
    input [4:0] r_da,
    input [5:0] g_da,
    input [4:0] b_da,
	input den,
	output dclk_m,
    output vsync_m,
    output hsync_m,
    output reg [3:0] da_m,
	 output den_m
    );
	 
    reg vsync_d1;
    reg vsync_d2;
    reg vsync_d3;
    reg hsync_d1;
    reg hsync_d2;
    reg hsync_d3;
	 reg den_d1;
	 reg den_d2;
	 reg den_d3;
	reg [1:0]data_sel;
	reg [15:0]da_hold;

always @ (posedge clk)
  if (rstb==1'b0) 
    begin
	   vsync_d1 <=1'b1;
	   vsync_d2 <=1'b1;
	   vsync_d3 <=1'b1;
	   hsync_d1 <=1'b1;
	   hsync_d2 <=1'b1;
	   hsync_d3 <=1'b1;
	   den_d1 <=1'b0;
	   den_d2 <=1'b0;
	   den_d3 <=1'b0;
    end
  else
    begin
	   vsync_d1 <=vsync;
	   vsync_d2 <=vsync_d1;
	   vsync_d3 <=vsync_d2;
	   hsync_d1 <=hsync;
	   hsync_d2 <=hsync_d1;
	   hsync_d3 <=hsync_d2;
	   den_d1 <=den;
	   den_d2 <=den_d1;
	   den_d3 <=den_d2;
    end
	 
always @ (posedge clk)
  if (rstb==1'b0) 
    data_sel <=2'd3;
  else
    if((den==1'b1)&&(den_d1==1'b0))
	   data_sel <=2'd0;
	 else if(den==1'b1)
	   data_sel <= data_sel +2'd1;
	 else
	   data_sel <= data_sel;
	 
always @ (posedge clk)
  if (rstb==1'b0)
    da_hold <=16'h0000;
  else
    if (data_sel==2'd3)
	  da_hold <={r_da[4:0],g_da[5:0],b_da[4:0]};
    else
      da_hold <= da_hold;
	  
always @ (posedge clk)
  if (rstb==1'b0)
    begin 
      da_m <=4'b0000;
    end
  else	 
	if(den_d2==1'b1)
	  case(data_sel)
	    2'd0:da_m <= da_hold[15:12];
	    2'd1:da_m <= da_hold[11:8];
	    2'd2:da_m <= da_hold[7:4];
	    2'd3:da_m <= da_hold[3:0];
	  endcase
	else
    begin 
      da_m <=4'b0000;
	 end
	
  //assign dclk_m = ~clk;	
  assign dclk_m = clk;	
  assign vsync_m = vsync_d2;	 
  assign hsync_m = hsync_d2;	 
  assign den_m = den_d2;
  
endmodule